import { UserService } from './../services/user.service';
import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { ButtonType } from '@gouvfr-anct/mediation-numerique/shared';
import { StructureWithOwners } from '../models/structureWithOwners.model';
import { User } from '../models/user.model';
import { StructureService } from '../services/structure.service';
import { ProfileService } from './services/profile.service';
import { Utils } from '../utils/utils';
import { catchError, map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { Location } from '@angular/common';

@Component({
  selector: 'app-profile',
  templateUrl: './profile.component.html',
  styleUrls: ['./profile.component.scss']
})
export class ProfileComponent implements OnInit {
  public userProfile: User;
  public structures: StructureWithOwners[] = [];
  public buttonTypeEnum = ButtonType;
  public isPublic: boolean;

  constructor(
    private profileService: ProfileService,
    private structureService: StructureService,
    private userService: UserService,
    private route: ActivatedRoute,
    private router: Router,
    private location: Location,
    public utils: Utils
  ) {}

  ngOnInit(): void {
    this.route.params.subscribe((urlParams) => {
      const userId = urlParams.id ? urlParams.id : '';
      if (userId) {
        this.isPublic = true;
        this.userService
          .getUser(userId)
          .pipe(
            map((res) => res),
            catchError(() => {
              this.router.navigate(['/home']);
              return new Observable<User>();
            })
          )
          .subscribe((user) => {
            this.userProfile = user;
            this.getStructuresFromProfile();
          });
      } else {
        this.isPublic = false;
        this.profileService.getProfile().then((profile: User) => {
          this.userProfile = profile;
          this.getStructuresFromProfile();
        });
      }
    });
  }

  private getStructuresFromProfile() {
    this.structures = [];
    this.userProfile.structuresLink.forEach((structureId) => {
      this.structureService.getStructureWithOwners(structureId, null).subscribe((s) => {
        this.structures.push(s);
      });
    });
  }

  public goBack(): void {
    this.location.back();
  }

  public addStructure(): void {
    this.router.navigateByUrl('/form/structure');
  }
}
