import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { Router } from '@angular/router';
import { Structure } from '@gouvfr-anct/mediation-numerique';
import { Owner } from '../../../models/owner.model';
import { User } from '../../../models/user.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { AuthService } from '../../../services/auth.service';
import { StructureService } from '../../../services/structure.service';

@Component({
  selector: 'app-structure-details-actions',
  templateUrl: 'structure-details-actions.component.html',
  styleUrls: ['./structure-details-actions.component.scss'],
})
export class StructureDetailsActionsComponent implements OnInit {
  @Input() public structure: Structure;

  @Output() public claimChange: EventEmitter<boolean> = new EventEmitter<boolean>();
  @Output() public deleteModal: EventEmitter<void> = new EventEmitter<void>();
  @Output() public pendingModal: EventEmitter<void> = new EventEmitter<void>();
  @Output() public claimModal: EventEmitter<void> = new EventEmitter<void>();
  @Output() public joinModal: EventEmitter<void> = new EventEmitter<void>();

  public currentProfile: User = null;
  public structureAdmins: Owner[] = [];
  public joinModalOpenned = false;
  public pendingModalOpenned = false;
  public claimModalOpenned = false;

  public constructor(
    public readonly profileService: ProfileService,
    private structureService: StructureService,
    private router: Router,
    private authService: AuthService
  ) {}

  public async ngOnInit(): Promise<void> {
    if (this.userIsLoggedIn()) {
      this.currentProfile = await this.profileService.getProfile();

      if (this.profileService.isAdmin()) {
        this.structureService.getStructureWithOwners(this.structure._id, this.currentProfile).subscribe((res) => {
          this.structureAdmins = res.owners;
        });
      }
    }
    this.claimChange.emit(await this.structureService.isClaimed(this.structure._id, this.currentProfile).toPromise());
  }

  public userIsLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }

  public togglePendingModal(): void {
    // this.pendingModalOpenned = !this.pendingModalOpenned;
    this.pendingModal.emit();
  }

  public toggleJoinModal(): void {
    // this.joinModalOpenned = !this.joinModalOpenned;
    this.joinModal.emit();
  }

  public toggleClaimModal(): void {
    // this.claimModalOpenned = !this.claimModalOpenned;
    this.claimModal.emit();
  }

  public handleJoin(): void {
    if (this.userIsLoggedIn()) {
      if (this.structure.isClaimed) {
        if (this.profileService.isPendingLinkedToStructure(this.structure._id)) {
          this.togglePendingModal();
        } else {
          this.toggleJoinModal();
        }
      } else {
        this.toggleClaimModal();
      }
    } else {
      this.router.navigate(['login'], { queryParams: { returnUrl: `acteurs?id=${this.structure._id}` } });
    }
  }

  public handleModify(): void {
    this.router.navigateByUrl(`/profile/edit-structure/${this.structure._id}`);
  }
}
