import { Component, EventEmitter, Input, Output } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Structure } from '@gouvfr-anct/mediation-numerique';
import { User } from '../../../models/user.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { AuthService } from '../../../services/auth.service';
import { StructureService } from '../../../services/structure.service';

@Component({
  selector: 'app-structure-details-modals',
  templateUrl: './structure-details-modals.component.html',
})
export class StructureDetailsModalsComponent {
  @Input() public structure: Structure;
  @Output() public claimChange: EventEmitter<boolean> = new EventEmitter<boolean>();

  public deleteModalOpenned = false;
  public claimModalOpenned = false;
  public joinModalOpenned = false;
  public pendingModalOpenned = false;

  public constructor(
    private profileService: ProfileService,
    private authService: AuthService,
    private route: ActivatedRoute,
    private router: Router,
    private structureService: StructureService
  ) {}

  public toggleClaimModal(): void {
    this.claimModalOpenned = !this.claimModalOpenned;
  }

  public toggleDeleteModal(): void {
    this.deleteModalOpenned = !this.deleteModalOpenned;
  }

  public toggleJoinModal(): void {
    this.joinModalOpenned = !this.joinModalOpenned;
  }

  public togglePendingModal(): void {
    this.pendingModalOpenned = !this.pendingModalOpenned;
  }

  private reload(): void {
    this.router.routeReuseStrategy.shouldReuseRoute = () => false;
    this.router.onSameUrlNavigation = 'reload';
    this.router.navigate(['./'], { relativeTo: this.route });
  }

  public claimStructure(shouldClaim: boolean): void {
    this.toggleClaimModal();
    if (shouldClaim) {
      this.structureService
        .claimStructureWithAccount(this.structure._id, this.authService.userValue.username)
        .subscribe();
      this.router.navigate(['join', this.structure._id], { state: { isClaimed: this.structure.isClaimed } });
    }
  }

  public deleteStructure(shouldDelete: boolean): void {
    this.toggleDeleteModal();
    if (shouldDelete) {
      this.structureService.delete(this.structure._id).subscribe((_res) => {
        this.reload();
      });
    }
  }

  public joinStructure(shouldClaim: boolean): void {
    this.toggleJoinModal();
    if (shouldClaim) {
      this.structureService.joinStructure(this.structure._id, this.authService.userValue.username).subscribe((_res) => {
        this.profileService.getProfile().then((_user: User) => {
          this.claimChange.emit(true);
        });
      });
    }
  }
}
