import { Component, Input, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Structure } from '@gouvfr-anct/mediation-numerique';
import { BehaviorSubject } from 'rxjs';
import { ParametersService } from '../../../admin/services/parameters.service';
import { Owner } from '../../../models/owner.model';
import { User } from '../../../models/user.model';
import { ProfileService } from '../../../profile/services/profile.service';
import { AuthService } from '../../../services/auth.service';
import { StructureService } from '../../../services/structure.service';
import { Utils } from '../../../utils/utils';

type StructureClaim = { [structureId: string]: boolean };

@Component({
  selector: 'app-structure-details-wrapper',
  templateUrl: './structure-details-wrapper.component.html',
  styleUrls: ['./structure-details-wrapper.component.scss'],
})
export class StructureDetailsWrapperComponent implements OnInit {
  @Input() public structure: Structure;
  public lockdownInfoDisplay = false;
  private _structureClaimsState: StructureClaim = {};
  private _structureClaims: BehaviorSubject<StructureClaim> = new BehaviorSubject({});
  public structureAdmins: Owner[] = [];
  constructor(
    private route: ActivatedRoute,
    private structureService: StructureService,
    private parametersService: ParametersService,
    private profileService: ProfileService,
    private authService: AuthService
  ) {
    this.parametersService.getParameters().subscribe((params) => {
      this.lockdownInfoDisplay = params.lockdownInfoDisplay;
    });
  }

  ngOnInit(): void {
    this.route.queryParams.subscribe((queryParams) => {
      if (queryParams.id) {
        this.structureService.getStructure(queryParams.id).subscribe((structure) => {
          this.structure = new Structure(structure);
          this.initData();
        });
      }
    });
  }

  async initData(): Promise<void> {
    let currentProfile = null;
    if (this.userIsLoggedIn()) {
      currentProfile = await this.profileService.getProfile();
      this.structureService.getStructureWithOwners(this.structure._id, currentProfile).subscribe((res) => {
        this.structureAdmins = res.owners;
      });
    }
    this.structure.isClaimed = await this.structureService.isClaimed(this.structure._id, currentProfile).toPromise();
  }

  public updateClaim(structureId: string, isClaim: boolean): void {
    this._structureClaimsState[structureId] = isClaim;
    this._structureClaims.next(this._structureClaimsState);
  }

  public userIsLoggedIn(): boolean {
    return this.authService.isLoggedIn();
  }

  public displayJobEmployer(profile: User): string {
    return new Utils().getJobEmployer(profile);
  }
}
